module Elasticsearch
  module Model
    module Adapter

      # An adapter for JsonApiResource-based models
      #
      module JsonApiResource

        Adapter.register self,
                         lambda { |klass| !!defined?(::JsonApiResource::Resource) && klass.respond_to?(:ancestors) && klass.ancestors.include?(::JsonApiResource::Resource) }

        module Records
          attr_writer :options

          def options
            @options ||= {}
          end

          # Returns an array of `JsonApiResource::Resource`
          #
          def records
            klass.where(id: ids, skip_pagination: true, order: "field(id, #{ids.join(", ")})")
          end
        end

        module Callbacks

          # Handle index updates (creating, updating or deleting documents)
          # when the model changes, by hooking into the lifecycle
          #
          # @see http://guides.rubyonrails.org/active_record_callbacks.html
          #
          def self.included(base)
            base.class_eval do
              after_commit lambda { __elasticsearch__.index_document  },  on: :create
              after_commit lambda { __elasticsearch__.update_document },  on: :update
              after_commit lambda { __elasticsearch__.delete_document },  on: :destroy
            end
          end
        end

        module Importing

          # Fetch batches of records from the database (used by the import method)
          #
          #
          # @see http://api.rubyonrails.org/classes/AcriveRecord/Batches.html AcriveRecord::Batches.find_in_batches
          #
          def __find_in_batches(options={}, &block)
            page = 1
            loop do
              objects = where page: page, per_page: 100
              
              yield objects

              page +=1

              break if page >= objects.meta[:total_pages]

            end
          end

          def __transform
            lambda { |model|  { index: { _id: model.id, data: model.__elasticsearch__.as_indexed_json } } }
          end
        end
      end
    end
  end
end
